package sitebuilder

import (
	"context"
	"errors"
	"lmika.dev/lmika/hugo-crm/models"
	"lmika.dev/lmika/hugo-crm/providers/db"
	"lmika.dev/lmika/hugo-crm/providers/git"
	"lmika.dev/lmika/hugo-crm/providers/hugo"
	"lmika.dev/lmika/hugo-crm/providers/netlify"
	"lmika.dev/lmika/hugo-crm/providers/themes"
	"log"
	"os"
)

type Service struct {
	db      *db.DB
	themes  *themes.Provider
	git     *git.Provider
	hugo    *hugo.Provider
	netlify *netlify.Provider
}

func New(
	db *db.DB,
	themes *themes.Provider,
	git *git.Provider,
	hugo *hugo.Provider,
	netlify *netlify.Provider,
) *Service {
	return &Service{
		db:      db,
		themes:  themes,
		git:     git,
		hugo:    hugo,
		netlify: netlify,
	}
}

func (s *Service) CreateNewSite(site models.Site) models.Job {
	return models.Job{
		Do: func(ctx context.Context) error {
			return s.createSite(ctx, site)
		},
	}
}

func (s *Service) RebuildSite(site models.Site) models.Job {
	return models.Jobs(
		models.Job{
			Do: func(ctx context.Context) error {
				// Teardown the existing site
				siteDir := s.hugo.SiteStagingDir(site, hugo.BaseSiteDir)
				if err := os.RemoveAll(siteDir); err != nil {
					return err
				}
				return nil
			},
		},
		s.CreateNewSite(site),
		s.WriteAllPosts(site),
		s.Publish(site),
	)
}

func (s *Service) createSite(ctx context.Context, site models.Site) error {
	themeMeta, ok := s.themes.Lookup(site.Theme)
	if !ok {
		return errors.New("theme not found")
	}

	// Build the site
	log.Printf(" .. build")
	if err := s.hugo.NewSite(ctx, site); err != nil {
		return err
	}

	// Setup the theme
	log.Printf(" .. theme")
	if err := s.git.Clone(ctx, themeMeta.URL, s.hugo.SiteStagingDir(site, hugo.ThemeSiteDir)); err != nil {
		return err
	}

	if err := s.hugo.ReconfigureSite(ctx, site); err != nil {
		return err
	}
	return nil
}
